#ifndef _HASHTAB_H
#define _HASHTAB_H

/*
 * 64 bit FNV-1a non-zero initial basis
 *
 * see http://www.isthe.com/chongo/tech/comp/fnv/index.html
 */
#define FNV1A_64_INIT ((u_int64_t)0xcbf29ce484222325ULL)
#if !defined(NO_FNV_GCC_OPTIMIZATION)
#define FNV_64_PRIME ((u_int64_t)0x100000001b3ULL)
#endif /* !NO_FNV_GCC_OPTIMIZATION */


typedef struct hnode *HNODEPTR;            /* site node (host) pointer     */
typedef struct unode *UNODEPTR;            /* url node pointer             */
typedef struct rnode *RNODEPTR;            /* referrer node                */
typedef struct anode *ANODEPTR;            /* user agent node pointer      */
typedef struct snode *SNODEPTR;            /* Search string node pointer   */
typedef struct inode *INODEPTR;            /* user (ident) node pointer    */
#ifdef USE_DNS
typedef struct dnode *DNODEPTR;            /* DNS hash table node struct   */
#endif
typedef struct pnode *PNODEPTR;		   /* URL dir path pointer         */

/* Object flags */
#define OBJ_REG  0                         /* Regular object               */
#define OBJ_HIDE 1                         /* Hidden object                */
#define OBJ_GRP  2                         /* Grouped object               */

#ifdef USE_DNS
struct dnode {  char *string;              /* DNS node hash table struct   */
              struct in_addr  addr;
              struct dnode    *llist;
              struct dnode    *next; };
#endif

struct hnode {  char *string;              /* host hash table structure    */
                 int flag;
              u_int64_t count;
              u_int64_t files;
              u_int64_t visit;                /* visit information            */
              u_int64_t tstamp;
                char *lasturl;
              u_int64_t xfer;
              struct hnode *next; };

struct unode {  char *string;              /* url hash table structure     */
                 int flag;                 /* Object type (REG, HIDE, GRP) */
              u_int64_t count;                /* requests counter             */
              u_int64_t files;                /* files counter                */
              u_int64_t entry;                /* entry page counter           */
              u_int64_t exit;                 /* exit page counter            */
              u_int64_t xfer;                 /* xfer size in bytes           */
              struct unode *next; };       /* pointer to next node         */

struct rnode {  char *string;              /* referrer hash table struct   */
                 int flag;
              u_int64_t count;
              struct rnode *next; };

struct anode {  char *string;
                 int flag;
              u_int64_t count;
              struct anode *next; };

struct snode {  char *string;                 /* search string struct      */
              u_int64_t count;
              struct snode *next; };

struct inode {  char *string;                 /* host hash table struct    */
                 int flag;
              u_int64_t count;
              u_int64_t files;
              u_int64_t visit;
              u_int64_t tstamp;
              u_int64_t xfer;
              struct inode *next; };

struct pnode { char *string;		   /* dirname of URL path component */
                int flag;                  /* Object type (REG or HIDE) */
              u_int64_t count;                /* requests counter            */
              u_int64_t xfer;                 /* xfer size in bytes           */
              struct pnode    *next; };

extern HNODEPTR sm_htab[MAXHASH];             /* hash tables               */
extern HNODEPTR sd_htab[MAXHASH];
extern UNODEPTR um_htab[MAXHASH];             /* for hits, sites,          */
extern RNODEPTR rm_htab[MAXHASH];             /* referrers and agents...   */
extern ANODEPTR am_htab[MAXHASH];
extern SNODEPTR sr_htab[MAXHASH];             /* search string table       */
extern INODEPTR im_htab[MAXHASH];             /* ident table (username)    */
#ifdef USE_DNS
extern DNODEPTR host_table[MAXHASH];          /* DNS resolver table        */
#endif
extern PNODEPTR di_htab[MAXHASH];             /* for URL dir stats         */

extern int    put_hnode(char *, int, u_int64_t, u_int64_t, u_int64_t,
			u_int64_t *, u_int64_t, u_int64_t, char *, HNODEPTR *);
extern int    put_unode(char *, int, u_int64_t, u_int64_t, u_int64_t *,
                        u_int64_t, u_int64_t, UNODEPTR *);
extern int    put_inode(char *, int, u_int64_t, u_int64_t, u_int64_t,
			u_int64_t *, u_int64_t, u_int64_t, INODEPTR *);
extern int    put_rnode(char *, int, u_int64_t, u_int64_t *, RNODEPTR *);
extern int    put_anode(char *, int, u_int64_t, u_int64_t *, ANODEPTR *);
extern int    put_snode(char *, u_int64_t, SNODEPTR *);
extern int    put_pnode(char *, int, u_int64_t, u_int64_t, u_int64_t *,
                        PNODEPTR *);

#ifdef USE_DNS
extern int    put_dnode(char *, struct in_addr *, DNODEPTR *);
extern void   del_dlist(DNODEPTR *);
#endif

extern void   del_htabs();                    /* delete hash tables        */
extern void   del_hlist(HNODEPTR *);          /* delete host htab          */
extern void   del_ulist(UNODEPTR *);          /* delete url htab           */
extern void   del_rlist(RNODEPTR *);          /* delete referrer htab      */
extern void   del_alist(ANODEPTR *);          /* delete host htab          */
extern void   del_slist(SNODEPTR *);          /* delete host htab          */
extern void   del_ilist(INODEPTR *);          /* delete host htab          */
extern void   del_plist(PNODEPTR *);          /* delete dir htab           */

extern void   month_update_exit(u_int64_t);
extern u_int64_t tot_visit(HNODEPTR *);
extern char   *find_url(char *);

#endif  /* _HASHTAB_H */
